#ifndef LSVPD_DEVICE_SCSI_H
#define LSVPD_DEVICE_SCSI_H

/*
  lsvpd - SCSI Device-related definitions for the lsvpd package.
  
  This file is part of the Linux lsvpd package.
  
  (C) Copyright IBM Corp. 2002, 2003, 2004, 2005, 2006
  
  Maintained by  Martin Schwenke <martins@au.ibm.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
 
  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
  $Id: device_scsi.h,v 1.1 2006/04/11 18:38:28 emunson Exp $
*/

#include <stdbool.h>

#include "device.h"
  
#define DEVICE_TYPE_SCSI "scsi"

/* These types use the SCSI layer too. */
#define DEVICE_TYPE_SATA       "sata"
#define DEVICE_TYPE_ATA        "ata"
#define DEVICE_TYPE_ISCSI      "iscsi"
#define DEVICE_TYPE_FIBRE      "fibre"
#define DEVICE_TYPE_RAID       "raid"
#define DEVICE_TYPE_SCSI_DEBUG "scsi_debug"
#define DEVICE_TYPE_IBMVSCSI   "ibmvscsi"
#define DEVICE_TYPE_USB        "usb"

extern struct device_scsi_ds_foo {
	const node_t type;
	const char *   ds_prefix;
} device_scsi_ds_prefixes[];

typedef struct device_scsi_address {
	unsigned int host;
	unsigned int bus;
	unsigned int target;
	unsigned int lun;
} device_scsi_address_t;

extern bool
device_parse_scsi_address(const char * s,
			  device_scsi_address_t * a);

extern bool
device_parse_scsi_bus_info(const bus_info_t bus_info,
			   device_scsi_address_t * a);

extern const char *
device_scsi_subtype_string(unsigned char subtype);

extern char *
device_scsi_ds(const node_t  type,
	       unsigned char subtype);

extern void
device_scsi_ata_mf_tm(const node_t type,
		      char ** mf,
		      char ** tm);

extern bool
device_scsi_append_field(vpd_list_t **         vpd,
			 const char *          key,
			 const unsigned char * data,
			 size_t                len,
			 const char *          subtype_string);
extern bool
device_scsi_append_field_string(vpd_list_t **  vpd,
				const char *   key,
				const char *   data,
				const char *   subtype_string);

extern void
device_scsi_vpd_sort(vpd_list_t ** vpd);

extern void
device_scsi_vpd_function_append(node_retrieve_vpd_function_t * f);

extern device_get_info_function_t * device_scsi_get_info_function;

extern device_get_info_function_t device_scsi_get_info;

#endif
