#ifndef _VPD_H_
#define _VPD_H_

/*
  vpd - Utilities for in memory and disk-based VPD handling.
  
  This file is part of the Linux lsvpd package.
  
  (C) Copyright IBM Corp. 2002, 2003, 2004, 2005
  
  Maintained by  Martin Schwenke <martins@au.ibm.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
 
  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
  $Id: vpd.h,v 1.1 2006/04/11 18:38:28 emunson Exp $
*/
  
#include <stdbool.h>
#include <sys/types.h>

#include "growbuf.h"

typedef unsigned char vpd_key_t[2];

typedef struct vpd_list {
	vpd_key_t         key;
	char *            val;
	struct vpd_list * prev;
	struct vpd_list * next;
} vpd_list_t;

typedef struct vpd_list_list {
	vpd_list_t *           vpd_list;
	struct vpd_list_list * prev;
	struct vpd_list_list * next;
} vpd_list_list_t;

extern vpd_list_t *
vpd_list_new(void);

extern void
vpd_list_free(vpd_list_t * vl);

extern vpd_list_list_t *
vpd_list_list_new(void);

extern void
vpd_list_list_free(vpd_list_list_t * vll);

extern void
vpd_list_append(vpd_list_t ** vl,
		vpd_list_t *  new);

extern void
vpd_list_prepend(vpd_list_t ** vl,
		 vpd_list_t *  new);

extern bool
vpd_list_append_key_value(vpd_list_t **   vl,
			  const vpd_key_t k,
			  const unsigned char * v,
			  size_t          vlen);

extern bool
vpd_list_prepend_key_value(vpd_list_t **   vl,
			   const vpd_key_t k,
			   const unsigned char * v,
			   size_t          vlen);

extern bool
vpd_list_maybe_append_key_string(vpd_list_t **   vpd,
				 const char *    k,
				 const char *    v);

extern void
vpd_list_sort(vpd_list_t ** vpd,
	      const char *  order[]);

extern vpd_list_list_t *
vpd_list_list_append(vpd_list_list_t * vll,
		     vpd_list_t *      vl);

extern bool
vpd_list_to_text(const vpd_list_t * vl,
		 growbuf **         buf);

extern growbuf *
vpd_list_list_to_text(const vpd_list_list_t * vll);

extern bool
vpd_list_to_vpd_dir(const vpd_list_t * vl,
		    const char *       vpd_dir);

extern bool
vpd_list_list_to_node(const vpd_list_list_t * vll,
		      const char *            node);

extern vpd_list_t *
vpd_text_to_vpd_list(const growbuf * buf);

extern vpd_list_list_t *
vpd_text_to_vpd_list_list(const growbuf * buf,
			  const vpd_key_t sep);

extern char *
vpd_dir_get_subdirnum(const char * dir,
		      int          subdirnum);

extern char *
vpd_dir_get_default(const char * dir);

extern bool
vpd_dir_field_ensure(const char *    vpd_dir,
		     const vpd_key_t field,
		     const char *    value);

#endif
